/*
 * Copyright (c) 2018 by iTAS Innovations Pvt. Ltd.

 * All rights reserved.
 *
 *          %name: cyfxmain.c %
 *       %version: 1 %
 * %date_modified: wed Dec 19 11:18:57 2018 %
 *    %derived_by: itasin %
 *          Owner: itasin
 *          Author: Thrimurthi M
 */

/* This file illustrates the Slave TS Write and I2C application example using the DMA AUTO mode. */

/*
   This example illustrates a Slave TS Write and I2C Read and write mechanism between P3 chip and USB end points. The example comprises of
   vendor class USB enumeration descriptors with 3 bulk end points.

   A I2C OUT end point acts as the producer of data from the host to P3 chip. A I2C IN end point acts as the consumer
   of data from P3 chip to the host. The I2C read and write data is achieved with the help of a DMA MANUAL_IN and DMA MANUAL_OUT channel.
   DMA MANUAL_IN and DMA MANUAL_OUT channel is created between the producer USB I2C end point and the consumer USB I2C end point.

   The GPIF configuration data for the Synchronous Slave FIFO Write operation is loaded onto
   the appropriate GPIF registers. The p-port data transfers are done via the producer
   p-port socket.

   The P to U DMA channel connects the producer p-port socket to the USB
   consumer (IN) end point.

   The DMA buffer size is defined based on the USB speed. 64 for full speed, 512 for high speed and 1024
   for super speed. CY_FX_I2CLP_DMA_BUF_COUNT in the header file defines the number of DMA buffers.
 */

#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3dma.h"
#include "cyu3error.h"
#include "cyfxmain.h"
#include "cyu3usb.h"
#include "cyu3uart.h"
#include "cyu3utils.h"
#include "cyu3i2c.h"
#include "cyu3spi.h"
#include <cyu3gpio.h>
#include "cyu3gpif.h"
#include "cyu3pib.h"
#include "cyu3utils.h"
#include "pib_regs.h"
#include "cyfxgpif2config.h"
#include "cyu3pib.h"
#include "i2c.h"
#include <cyu3socket.h>

CyU3PThread     P3LpAppThread;	 /* P3 loop application thread structure */
CyU3PDmaChannel glChHandleUtoI2C;   /* DMA Channel handle for U2I2C transfer. */
CyU3PDmaChannel glChHandleI2CtoU;   /* DMA Channel handle for I2C2U transfer. */
CyU3PDmaMultiChannel glChHandleSlFifoPtoU;   /* Multi DMA Channel handle for P2U transfer. */

// DAM Buffer for i2c transfer
CyU3PDmaBuffer_t inBuf_p, outBuf_p;

uint32_t glFIFODMARxCount = 0;           /* Counter to track the number of buffers received from GPIF-2. */
CyBool_t glIsApplnActive = CyFalse;      /* Whether the loop back application is active or not. */


/* Application Error Handler */
void
CyFxAppErrorHandler (
		CyU3PReturnStatus_t apiRetStatus    /* API return status */
)
{
	/* Application failed with the error code apiRetStatus */

	/* Add custom debug or recovery actions here */

	/* Loop Indefinitely */
	for (;;)
	{
		/* Thread sleep : 100 ms */
		CyU3PThreadSleep (100);
	}
}

/* This function initializes the debug module. The debug prints
 * are routed to the UART and can be seen using a UART console
 * running at 115200 baud rate. */
CyU3PReturnStatus_t
CyFxI2CLpApplnDebugInit (void)
{
	CyU3PUartConfig_t uartConfig;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	/* Initialize the UART for printing debug messages */
	apiRetStatus = CyU3PUartInit();
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		/* Error handling */
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Set UART configuration */
	CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
	uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
	uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
	uartConfig.parity = CY_U3P_UART_NO_PARITY;
	uartConfig.txEnable = CyTrue;
	uartConfig.rxEnable = CyFalse;
	uartConfig.flowCtrl = CyFalse;
	uartConfig.isDma = CyTrue;

	apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Set the UART transfer to a really large value. */
	apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Initialize the debug module. */
	apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyFxAppErrorHandler(apiRetStatus);
	}

	CyU3PDebugPreamble (CyFalse);

	return apiRetStatus;
}

/* I2c initialization for programming. */
CyU3PReturnStatus_t
CyFxI2cInit ()
{
	CyU3PI2cConfig_t i2cConfig;
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;


	/* Initialize and configure the I2C master module. */
	status = CyU3PI2cInit ();

	if (status != CY_U3P_SUCCESS)
	{
		return status;
	}

	/* Start the I2C master block. The bit rate is set at 400KHz.
	 * The data transfer is done via DMA. */
	CyU3PMemSet ((uint8_t *)&i2cConfig, 0, sizeof(i2cConfig));
	i2cConfig.bitRate    = CY_FX_USBI2C_I2C_BITRATE;
	i2cConfig.busTimeout = 0xFFFFFFFF;
	i2cConfig.dmaTimeout = 0xFFFF;
	i2cConfig.isDma      = CyFalse;

	status = CyU3PI2cSetConfig (&i2cConfig, NULL);
	if (status != CY_U3P_SUCCESS)
	{
		return status;
	}

	return status;
}

/* This function starts the P3 application. This is called
 * when a SET_CONF event is received from the USB host. The endpoints
 * are configured and the DMA pipe is setup in this function. */
void
CyFxP3LpApplnStart (
		void)
{
	uint16_t size = 0;
	CyU3PEpConfig_t epCfg;
	CyU3PDmaChannelConfig_t dmaCfg;
	CyU3PDmaMultiChannelConfig_t MultidmaCfg;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
	CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

	/* First identify the usb speed. Once that is identified,
	 * create a DMA channel and start the transfer on this. */

	/* Based on the Bus Speed configure the endpoint packet size */
	switch (usbSpeed)
	{
	case CY_U3P_FULL_SPEED:
		size = 64;
		break;

	case CY_U3P_HIGH_SPEED:
		size = 512;
		break;

	case  CY_U3P_SUPER_SPEED:
		size = 1024;
		break;

	default:
		CyU3PDebugPrint (4, "Error! Invalid USB speed.\n");
		CyFxAppErrorHandler (CY_U3P_ERROR_FAILURE);
		break;
	}

	CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
	epCfg.enable = CyTrue;
	epCfg.epType = CY_U3P_USB_EP_BULK;
	epCfg.burstLen = 1;
	epCfg.streams = 0;
	epCfg.pcktSize = size;


	// I2C Interface
	/* Producer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}

	/* Consumer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER_1, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}
	/* Create a DMA MANUAL IN channel for U2I2C transfer.
	 * DMA size is set based on the USB speed. */
	dmaCfg.size  = size;
	dmaCfg.count = CY_FX_I2CLP_DMA_BUF_COUNT;
	dmaCfg.prodSckId = CY_FX_EP_UtoI2C_PRODUCER_SOCKET;
	dmaCfg.consSckId = CY_U3P_CPU_SOCKET_CONS;
	dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
	/* Enabling the callback for produce event. */
	dmaCfg.notification = 0;
	dmaCfg.cb = NULL;
	dmaCfg.prodHeader = 0;
	dmaCfg.prodFooter = 0;
	dmaCfg.consHeader = 0;
	dmaCfg.prodAvailCount = 0;

	apiRetStatus = CyU3PDmaChannelCreate (&glChHandleUtoI2C,
			CY_U3P_DMA_TYPE_MANUAL_IN, &dmaCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "1stCyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Create a DMA MANUAL OUT channel for I2C2U transfer. */
	dmaCfg.prodSckId = CY_U3P_CPU_SOCKET_PROD;
	dmaCfg.consSckId = CY_FX_EP_I2CtoU_CONSUMER_SOCKET;
	dmaCfg.notification = 0;
	dmaCfg.cb = NULL;
	apiRetStatus = CyU3PDmaChannelCreate (&glChHandleI2CtoU,
			CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "2ndCyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Flush the Endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER_1);

	/* Set DMA channel transfer size. */
	apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleUtoI2C, CY_FX_I2CLP_DMA_TX_SIZE);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer Failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}
	apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleI2CtoU, CY_FX_I2CLP_DMA_TX_SIZE);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer Failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	// Slave FIFO Interface

#if 0
	epCfg.enable = CyTrue;
	epCfg.epType = CY_U3P_USB_EP_ISO;
	epCfg.burstLen = 15;
	epCfg.streams = 0;
	epCfg.pcktSize = size;
	epCfg.isoPkts=3;
#endif

	/* Consumer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER_2, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetTSEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}

	/* Create a DMA AUTO channel for P2U transfer.
	 * DMA size is set based on the USB speed. */
	MultidmaCfg.size  = (1024*CY_FX_DMA_BUF_SIZE);
	MultidmaCfg.count = CY_FX_FIFOLP_DMA_BUF_COUNT;
	MultidmaCfg.validSckCount = 2;
	MultidmaCfg.prodSckId[0] = (CyU3PDmaSocketId_t)CY_FX_PRODUCER_PPORT_SOCKET_0;
	MultidmaCfg.prodSckId[1] = (CyU3PDmaSocketId_t)CY_FX_PRODUCER_PPORT_SOCKET_1;
	MultidmaCfg.consSckId[0] = (CyU3PDmaSocketId_t)CY_FX_TS_CONSUMER_USB_SOCKET;
	MultidmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
	/* Enabling the callback for produce event. */
	MultidmaCfg.notification = 0;
	MultidmaCfg.cb = NULL;
	MultidmaCfg.prodHeader = 0;
	MultidmaCfg.prodFooter = 0;
	MultidmaCfg.consHeader = 0;
	MultidmaCfg.prodAvailCount = 0;

	apiRetStatus = CyU3PDmaMultiChannelCreate (&glChHandleSlFifoPtoU,
			CY_U3P_DMA_TYPE_AUTO_MANY_TO_ONE, &MultidmaCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaMultiChannelCreate failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}


	/* Flush the Endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER_2);

	apiRetStatus = CyU3PDmaMultiChannelSetXfer (&glChHandleSlFifoPtoU, CY_FX_FIFOLP_DMA_TX_SIZE, 0);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaMultiChannelSetXfer Failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	glIsApplnActive = CyTrue;
}

/* This function stops the P3 application. This shall be called whenever
 * a RESET or DISCONNECT event is received from the USB host. The end points are
 * disabled and the DMA pipe is destroyed by this function. */
void
CyFxP3LpApplnStop (
		void)
{
	CyU3PEpConfig_t epCfg;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	/* Update the flag. */
	glIsApplnActive = CyFalse;

	// I2C Interface
	/* Unregister the EP event callback. */
	CyU3PUsbRegisterEpEvtCallback (0, 0, 0, 0);

	/* Flush the endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER_1);

	/* Destroy the channel */
	CyU3PDmaChannelDestroy (&glChHandleUtoI2C);
	CyU3PDmaChannelDestroy (&glChHandleI2CtoU);


	/* Disable endpoints. */
	CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
	epCfg.enable = CyFalse;

	/* Producer endpoint configuration. */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}

	/* Consumer endpoint configuration. */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER_1, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}

	// Slave FIFO Interface
	/* Flush the endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER_2);

	/* Destroy the channel */
	CyU3PDmaMultiChannelDestroy (&glChHandleSlFifoPtoU);

	/* Disable endpoints. */
	CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
	epCfg.enable = CyFalse;

	/* Consumer endpoint configuration. */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER_2, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler (apiRetStatus);
	}
}


/* Callback to handle the USB setup requests. */
CyBool_t
CyFxLpApplnUSBSetupCB (
		uint32_t setupdat0, /* SETUP Data 0 */
		uint32_t setupdat1  /* SETUP Data 1 */
)
{
	/* Fast enumeration is used. Only requests addressed to the interface, class,
	 * vendor and unknown control requests are received by this function.
	 * This application does not support any class or vendor requests. */

	uint8_t  bRequest, bReqType;
	uint8_t  bType, bTarget;
	uint16_t wValue, wIndex;
	CyBool_t isHandled = CyFalse;

	/* Decode the fields from the setup request. */
	bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);
	bType    = (bReqType & CY_U3P_USB_TYPE_MASK);
	bTarget  = (bReqType & CY_U3P_USB_TARGET_MASK);
	bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
	wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);
	wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);

	if (bType == CY_U3P_USB_STANDARD_RQT)
	{
		/* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
		 * requests here. It should be allowed to pass if the device is in configured
		 * state and failed otherwise. */
		if ((bTarget == CY_U3P_USB_TARGET_INTF) && ((bRequest == CY_U3P_USB_SC_SET_FEATURE)
				|| (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) && (wValue == 0))
		{
			if (glIsApplnActive || glIsApplnActive)
				CyU3PUsbAckSetup ();
			else
				CyU3PUsbStall (0, CyTrue, CyFalse);

			isHandled = CyTrue;
		}

		/* CLEAR_FEATURE request for endpoint is always passed to the setup callback
		 * regardless of the enumeration model used. When a clear feature is received,
		 * the previous transfer has to be flushed and cleaned up. This is done at the
		 * protocol level. Since this is just a loopback operation, there is no higher
		 * level protocol. So flush the EP memory and reset the DMA channel associated
		 * with it. If there are more than one EP associated with the channel reset both
		 * the EPs. The endpoint stall and toggle / sequence number is also expected to be
		 * reset. Return CyFalse to make the library clear the stall and reset the endpoint
		 * toggle. Or invoke the CyU3PUsbStall (ep, CyFalse, CyTrue) and return CyTrue.
		 * Here we are clearing the stall. */
		if ((bTarget == CY_U3P_USB_TARGET_ENDPT) && (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)
				&& (wValue == CY_U3P_USBX_FS_EP_HALT))
		{
			if (glIsApplnActive)
			{
				if ((wIndex == CY_FX_EP_PRODUCER) || (wIndex == CY_FX_EP_CONSUMER_1))
				{

					CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyTrue);
					CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER_1, CyTrue);
					CyU3PBusyWait (125);

					CyU3PDmaChannelReset (&glChHandleUtoI2C);
					CyU3PDmaChannelReset (&glChHandleI2CtoU);

					CyU3PUsbFlushEp (CY_FX_EP_PRODUCER);
					CyU3PUsbFlushEp (CY_FX_EP_CONSUMER_1);
					CyU3PUsbResetEp (CY_FX_EP_PRODUCER);
					CyU3PUsbResetEp (CY_FX_EP_CONSUMER_1);
					CyU3PDmaChannelSetXfer (&glChHandleUtoI2C, CY_FX_I2CLP_DMA_TX_SIZE);
					CyU3PDmaChannelSetXfer (&glChHandleI2CtoU, CY_FX_I2CLP_DMA_TX_SIZE);

					CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyFalse);
					CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER_1, CyFalse);
				}
				if (wIndex == CY_FX_EP_CONSUMER_2)
				{
					CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER_2, CyTrue);
					CyU3PBusyWait (125);

					CyU3PDmaMultiChannelReset (&glChHandleSlFifoPtoU);

					CyU3PUsbFlushEp(CY_FX_EP_CONSUMER_2);
					CyU3PUsbResetEp (CY_FX_EP_CONSUMER_2);
					CyU3PDmaMultiChannelSetXfer (&glChHandleSlFifoPtoU, CY_FX_FIFOLP_DMA_TX_SIZE,0);

					CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER_2, CyFalse);
				}

				CyU3PUsbStall (wIndex, CyFalse, CyTrue);
				CyU3PUsbAckSetup ();
				isHandled = CyTrue;
			}
		}
	}

	return isHandled;
}

/* Callback function to check for PIB ERROR*/
void gpif_error_cb(CyU3PPibIntrType cbType, uint16_t cbArg)
{
	if(cbType==CYU3P_PIB_INTR_ERROR)
	{
		switch(CYU3P_GET_PIB_ERROR_TYPE(cbArg))
		{
		case CYU3P_PIB_ERR_THR0_WR_OVERRUN:
			CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR0_WR_OVERRUN\r\n");
			break;
		case CYU3P_PIB_ERR_THR1_WR_OVERRUN:
			CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR1_WR_OVERRUN\r\n");
			break;
		case CYU3P_PIB_ERR_THR2_WR_OVERRUN:
			CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR2_WR_OVERRUN\r\n");
			break;
		case CYU3P_PIB_ERR_THR3_WR_OVERRUN:
			CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR3_WR_OVERRUN\r\n");
			break;

		default:
			CyU3PDebugPrint (4, "GPIF ERROR CB_ERROR :%d\r\n",CYU3P_GET_PIB_ERROR_TYPE(cbArg));
			break;
		}
	}
}

/* GpifCB callback function is invoked when FV triggers GPIF interrupt */
void
CyFxGpifCB (
		uint8_t currentState            /* GPIF state which triggered the interrupt. */
)
{
	/* The ongoing video frame has ended. If we have a partial buffer sitting on the socket, we need to forcibly
	 * wrap it up. We also need to toggle the FW_TRG a couple of times to get the state machine ready for the
	 * next frame.
	 *
	 * Note: DMA channel APIs cannot be used here as this is ISR context. We are making use of the raw socket
	 * APIs.
	 */
	switch (currentState)
	{
#if 1
	case PARTIAL_BUF_IN_SCK0:
		CyU3PDmaSocketSetWrapUp (CY_U3P_PIB_SOCKET_0);
		break;
	case PARTIAL_BUF_IN_SCK1:
		CyU3PDmaSocketSetWrapUp (CY_U3P_PIB_SOCKET_1);
		break;
#endif

	case FULL_BUF_IN_SCK0:
			break;
	case FULL_BUF_IN_SCK1:
		break;

	default:
		/* This should not happen. Do nothing. */
		return;
	}

	CyU3PGpifControlSWInput (CyTrue);
	CyU3PGpifControlSWInput (CyFalse);
}

/* This is the callback function to handle the USB events. */
void
CyFxILpApplnUSBEventCB (
		CyU3PUsbEventType_t evtype, /* Event type */
		uint16_t            evdata  /* Event data */
)
{
	switch (evtype)
	{
	case CY_U3P_USB_EVENT_SETCONF:
		/* Disable the low power entry to optimize USB throughput */
		CyU3PUsbLPMDisable();
		/* Stop the application before re-starting. */
		if (glIsApplnActive)
		{
			CyFxP3LpApplnStop ();
		}
		/* Start the loop back function. */
		CyFxP3LpApplnStart ();
		break;

	case CY_U3P_USB_EVENT_RESET:
	case CY_U3P_USB_EVENT_DISCONNECT:
		/* Stop the loop back function. */
		if (glIsApplnActive)
		{
			CyFxP3LpApplnStop ();
		}
		break;

	default:
		break;
	}
}

/* Callback function to handle LPM requests from the USB 3.0 host. This function is invoked by the API
   whenever a state change from U0 -> U1 or U0 -> U2 happens. If we return CyTrue from this function, the
   FX3 device is retained in the low power state. If we return CyFalse, the FX3 device immediately tries
   to trigger an exit back to U0.

   This application does not have any state in which we should not allow U1/U2 transitions; and therefore
   the function always return CyTrue.
 */
CyBool_t
CyFxLpApplnLPMRqtCB (
		CyU3PUsbLinkPowerMode link_mode)
{
	return CyTrue;
}


/* This function initializes the GPIF interface and initializes
 * the USB interface. */
CyU3PReturnStatus_t
CyFxSlaveFifoApplnInit (void)
{
	CyU3PPibClock_t pibClock;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	/* Initialize the p-port block. */
	pibClock.clkDiv = 2;
	pibClock.clkSrc = CY_U3P_SYS_CLK;
	pibClock.isHalfDiv = CyFalse;
	/* Disable DLL for sync GPIF */
	pibClock.isDllEnable = CyFalse;
	apiRetStatus = CyU3PPibInit(CyTrue, &pibClock);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "P-port Initialization failed, Error Code = %d\n",apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Load the GPIF configuration for Slave FIFO sync mode. */
	apiRetStatus = CyU3PGpifLoad (&CyFxGpifConfig);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PGpifLoad failed, Error Code = %d\n",apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}


	/* Register the GPIF State Machine callback used to get frame end notifications.
	 * We use the fast callback version which is triggered from ISR context.
	 */
	CyU3PGpifRegisterSMIntrCallback (CyFxGpifCB);

	CyU3PGpifSocketConfigure (0,CY_U3P_PIB_SOCKET_0,6,CyFalse,1);

	/* Start the state machine. */
	apiRetStatus = CyU3PGpifSMStart (START_SCK0, ALPHA_START_SCK0);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PGpifSMStart failed, Error Code = %d\n",apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}
#if 0
	/* Register a callback for notification of PIB interrupts*/
	CyU3PPibRegisterCallback (gpif_error_cb, CYU3P_PIB_INTR_ERROR);
#endif
	return apiRetStatus;
}


/* This function initializes the USB Module, sets the enumeration descriptors.
 * This function does not start the TS streaming and this is done only when
 * SET_CONF event is received. */
CyU3PReturnStatus_t
CyFxP3LpApplnInit (void)
{
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	/* Start the USB functionality. */
	apiRetStatus = CyU3PUsbStart();
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PUsbStart failed to Start, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* The fast enumeration is the easiest way to setup a USB connection,
	 * where all enumeration phase is handled by the library. Only the
	 * class / vendor requests need to be handled by the application. */
	CyU3PUsbRegisterSetupCallback(CyFxLpApplnUSBSetupCB, CyTrue);

	/* Register a callback to handle LPM requests from the USB 3.0 host. */
	CyU3PUsbRegisterLPMRequestCallback(CyFxLpApplnLPMRqtCB);

	/* Setup the callback to handle the USB events. */
	CyU3PUsbRegisterEventCallback(CyFxILpApplnUSBEventCB);

	/* Set the USB Enumeration descriptors */

	/* Super speed device descriptor. */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB30DeviceDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* High speed device descriptor. */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB20DeviceDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* BOS descriptor */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0, (uint8_t *)CyFxUSBBOSDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set configuration descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Device qualifier descriptor */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0, (uint8_t *)CyFxUSBDeviceQualDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set device qualifier descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Super speed configuration descriptor */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBSSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set configuration descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* High speed configuration descriptor */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBHSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB Set Other Speed Descriptor failed, Error Code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Full speed configuration descriptor */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBFSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB Set Configuration Descriptor failed, Error Code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* String descriptor 0 */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxUSBStringLangIDDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* String descriptor 1 */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBManufactureDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* String descriptor 2 */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Connect the USB Pins with super speed operation enabled. */
	apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "USB Connect failed, Error code = %d\n", apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}
	return apiRetStatus;
}

/* Init the GPIO module */
CyU3PReturnStatus_t CyFxUsbGPIOInit ()
{
	CyU3PGpioClock_t gpioClock;
	CyU3PGpioSimpleConfig_t gpioConfig;
	CyU3PReturnStatus_t Status = CY_U3P_SUCCESS;


	/* Init the GPIO module */
	gpioClock.fastClkDiv = 2;
	gpioClock.slowClkDiv = 0;
	gpioClock.simpleDiv = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
	gpioClock.clkSrc = CY_U3P_SYS_CLK;
	gpioClock.halfDiv = 0;

	Status = CyU3PGpioInit(&gpioClock, NULL);
	if (Status != 0)
	{
		/* Error Handling */
		CyU3PDebugPrint (4, "CyU3PGpioInit failed, error code = %d\n", Status);
	}

	/* Configure GPIO 23 as P3 RSTt */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	Status = CyU3PGpioSetSimpleConfig(Demod_RST, &gpioConfig);
	if (Status != CY_U3P_SUCCESS)
	{
		/* Error handling */
		CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig failed, error code = %d\n",Status);
	}
	/* Configure GPIO 27 as Tuner GPIO */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	Status = CyU3PGpioSetSimpleConfig(Tuner_GPIO, &gpioConfig);
	if (Status != CY_U3P_SUCCESS)
	{
		/* Error handling */
		CyU3PDebugPrint (4, "CyU3PGpioSetSimpleConfig failed, error code = %d\n",Status);
	}
	// Make Tuner_GPIO Pin is Low
	CyU3PGpioSimpleSetValue (Tuner_GPIO,CyFalse);
	return Status;
}


CyU3PReturnStatus_t End_Point_In_Buff()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	/* Wait for receiving a buffer from the producer socket (OUT endpoint). The call
	 * will fail if there was an error or if the USB connection was reset / disconnected.
	 * In case of error invoke the error handler and in case of reset / disconnection,
	 * glIsApplnActive will be CyFalse; continue to beginning of the loop. */

	status = CyU3PDmaChannelGetBuffer (&glChHandleUtoI2C, &inBuf_p, CYU3P_WAIT_FOREVER);
	if (status != CY_U3P_SUCCESS)
	{
		return status;
	}
	CyU3PMemCopy (copyBuffer,inBuf_p.buffer, inBuf_p.count);


#if 0
	for(int y=0;y<inBuf_p.count; y++)
	{
		CyU3PDebugPrint (4, "copyBuffer[%d] = 0x%x\r\n",y,copyBuffer[y]);
	}
#endif
	CyU3PDebugPrint (4, "get_in_inBuf_p.count = %d\n", inBuf_p.count);

	/* Now discard the data from the producer channel so that the buffer is made available
	 * to receive more data. */
	status = CyU3PDmaChannelDiscardBuffer (&glChHandleUtoI2C);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaChannelDiscardBuffer failed, Error code = %d\n", status);
		CyFxAppErrorHandler(status);
	}
	return status;
}


void End_Point_Out_Buff(uint32_t OutBuf_count)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	/* Wait for a free buffer to transmit the I2C Read data data */
	status = CyU3PDmaChannelGetBuffer (&glChHandleI2CtoU, &outBuf_p, CYU3P_WAIT_FOREVER);
	if (status != CY_U3P_SUCCESS)
	{
}

	CyU3PMemCopy (outBuf_p.buffer, copyBuffer, OutBuf_count);
#if 0
	for(int x=0;x<OutBuf_count; x++)
	{
		CyU3PDebugPrint (4, "outBuf_p.buffer[%d] = 0x%x\r\n",x,outBuf_p.buffer[x]);
	}
#endif
	CyU3PDebugPrint (4, "outBuf_p.count = %d\n", OutBuf_count);

	/* Commit the I2C Read data to the consumer pipe so that the data can be
	   transmitted back to the USB host */
	status = CyU3PDmaChannelCommitBuffer (&glChHandleI2CtoU, OutBuf_count, 0);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint (4, "CyU3PDmaChannelCommitoutBuffer failed, Error code = %d\n", status);
		CyFxAppErrorHandler(status);
	}
}

/* Entry function for the P3LpAppThread. */
void
P3LpAppThread_Entry (
		uint32_t input)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	/* Initialize the debug module */
	status = CyFxI2CLpApplnDebugInit();

	if (status != CY_U3P_SUCCESS)
	{
		goto handle_error;
	}

	CyU3PDebugPrint (1, "UART initialized\r\n");

	/* Initialize the application. */
	status = CyFxI2cInit ();
	if (status != CY_U3P_SUCCESS)
	{
		goto handle_error;
	}
	CyU3PDebugPrint (1, "I2C initialized\r\n");

	/* Initialize the slave FIFO application */
	status = CyFxSlaveFifoApplnInit();
	if (status != CY_U3P_SUCCESS)
	{
		goto handle_error;
	}
	CyU3PDebugPrint (1, "Salve FIFO initialized\r\n");

	/* Initialize the GPIO Reset */

	status = CyFxUsbGPIOInit ();

	if (status != CY_U3P_SUCCESS)
	{
		goto handle_error;
	}
	CyU3PDebugPrint (1, "GPIO initialized\r\n");

	/* Initialize the I2C loop application */
	status =CyFxP3LpApplnInit();

	if (status != CY_U3P_SUCCESS)
	{
		goto handle_error;
	}

	for (;;)
	{

		if (glIsApplnActive)
		{
			TD_Poll();
		}
		else
			CyU3PThreadSleep (500);
	}

	handle_error: CyU3PDebugPrint (4, "%x: Application failed to initialize. Error code: %d.\n", status);
	while (1);
}

/* Application define function which creates the threads. */
void
CyFxApplicationDefine (
		void)
{
	void *ptr1;
	uint32_t retThrdCreate = CY_U3P_SUCCESS;

	/* Allocate the memory for the threads */
	ptr1 = CyU3PMemAlloc (CY_FX_P3LP_THREAD_STACK);

	/* Create the thread for the application */
	retThrdCreate = CyU3PThreadCreate (&P3LpAppThread,           /* P3 loop App Thread structure */
			"21:P3 Firmware",                   /* Thread ID and Thread name */
			P3LpAppThread_Entry,                   /* P3 loop App Thread Entry function */
			0,                                       /* No input parameter to thread */
			ptr1,                                     /* Pointer to the allocated thread stack */
			CY_FX_P3LP_THREAD_STACK,               /* P3 loop App Thread stack size */
			CY_FX_P3LP_THREAD_PRIORITY,            /* P3 loop App Thread priority */
			CY_FX_P3LP_THREAD_PRIORITY,            /* P3 loop App Thread priority */
			CYU3P_NO_TIME_SLICE,                     /* No time slice for the application thread */
			CYU3P_AUTO_START                         /* Start the Thread immediately */
	);

	/* Check the return code */
	if (retThrdCreate != 0)
	{
		/* Thread Creation failed with the error code retThrdCreate */

		/* Add custom recovery or debug actions here */

		/* Application cannot continue */
		/* Loop indefinitely */
		while(1);
	}

}


/*
 * Main function
 */
int
main (void)
{
	CyU3PIoMatrixConfig_t io_cfg;
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PSysClockConfig_t clockConfig;

	/* When the GPIF data bus is configured as 32-bits wide and running at 100 MHz (synchronous),
       the FX3 system clock has to be set to a frequency greater than 400 MHz. */

	clockConfig.setSysClk400  = CyFalse;
	clockConfig.cpuClkDiv     = 2;
	clockConfig.dmaClkDiv     = 2;
	clockConfig.mmioClkDiv    = 2;
	clockConfig.useStandbyClk = CyFalse;
	clockConfig.clkSrc        = CY_U3P_SYS_CLK;
	/* Initialize the device */
	status = CyU3PDeviceInit (&clockConfig);
	if (status != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	/* Initialize the caches. Enable both Instruction and Data Caches. */
	status = CyU3PDeviceCacheControl (CyTrue, CyTrue, CyTrue);
	if (status != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	/* Configure the IO matrix for the device. On the FX3 DVK board, the COM port
	 * is connected to the IO(53:56). This means that either DQ32 mode should be
	 * selected or lppMode should be set to UART_ONLY. Here we are choosing
	 * UART_ONLY configuration. */
	io_cfg.isDQ32Bit = CyFalse;
	io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
	io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
	io_cfg.useUart   = CyTrue;
	io_cfg.useI2C    = CyTrue;
	io_cfg.useI2S    = CyFalse;
	io_cfg.useSpi    = CyFalse;
	io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_UART_ONLY;
	/* GPIO 23 are enabled. */
	io_cfg.gpioSimpleEn[0]  = 0x08800000;
	io_cfg.gpioSimpleEn[1]  = 0x00000000;
	io_cfg.gpioComplexEn[0] = 0;
	io_cfg.gpioComplexEn[1] = 0;
	status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
	if (status != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	/* This is a non returnable call for initializing the RTOS kernel */
	CyU3PKernelEntry ();

	/* Dummy return to make the compiler happy */
	return 0;

	handle_fatal_error:

	/* Cannot recover from this error. */
	while (1);
}

/* [ ] */

